@@ -1,4 +1,4 @@
-# Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,7 +10,7 @@ package Devel::Cover::Branch;
 use strict;
 use warnings;
 
-our $VERSION = "0.59";
+our $VERSION = "0.66";
 
 use base "Devel::Cover::Criterion";
 
@@ -54,10 +54,10 @@ sub calculate_summary
     my $s = $db->{summary};
     $self->pad;
 
-    $self->aggregate($s, $file, 'total', $self->total);
-    $self->aggregate($s, $file, 'uncoverable', $self->uncoverable);
-    $self->aggregate($s, $file, 'covered', $self->covered);
-    $self->aggregate($s, $file, 'error', $self->error);
+    $self->aggregate($s, $file, "total",       $self->total      );
+    $self->aggregate($s, $file, "uncoverable", $self->uncoverable);
+    $self->aggregate($s, $file, "covered",     $self->covered    );
+    $self->aggregate($s, $file, "error",       $self->error      );
 }
 
 1
@@ -88,11 +88,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.59 - 23rd August 2006
+Version 0.66 - 12th April 2010
 
 =head1 LICENCE
 
-Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
