@@ -1,4 +1,4 @@
-# Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,11 +10,11 @@ package Devel::Cover::DB;
 use strict;
 use warnings;
 
-our $VERSION = "0.59";
+our $VERSION = "0.66";
 
-use Devel::Cover::Criterion     0.59;
-use Devel::Cover::DB::File      0.59;
-use Devel::Cover::DB::Structure 0.59;
+use Devel::Cover::Criterion     0.66;
+use Devel::Cover::DB::File      0.66;
+use Devel::Cover::DB::Structure 0.66;
 
 use Carp;
 use File::Path;
@@ -27,16 +27,18 @@ my $DB = "cover.12";  # Version 12 of the database.
 @Devel::Cover::DB::Criteria_short =
     (qw( stmt      bran   path cond      sub        pod time ));
 
+# use Data::Dumper; $Data::Dumper::Indent = 1; $Data::Dumper::Sortkeys = 1;
+
 sub new
 {
     my $class = shift;
     my $self  =
     {
-        criteria       => \@Devel::Cover::DB::Criteria,
-        criteria_short => \@Devel::Cover::DB::Criteria_short,
-        runs           => {},
-        collected      => {},
-        uncoverable    => [],
+        criteria         => \@Devel::Cover::DB::Criteria,
+        criteria_short   => \@Devel::Cover::DB::Criteria_short,
+        runs             => {},
+        collected        => {},
+        uncoverable_file => [],
         @_
     };
 
@@ -45,17 +47,13 @@ sub new
     $self->{base} ||= $self->{db};
     bless $self, $class;
 
-    my $file;
     if (defined $self->{db})
     {
         $self->validate_db;
-        $file = "$self->{db}/$DB";
+        my $file = "$self->{db}/$DB";
         $self->read($file) if -e $file;
-        return $self unless -e $file;
     }
 
-    # croak "No input db, filehandle or cover" unless defined $self->{cover};
-
     $self
 }
 
@@ -106,6 +104,9 @@ sub delete
     $self->{db} = $db if ref $self;
     croak "No db specified" unless length $db;
 
+    return $self unless -d $db;
+
+    # TODO - just delete the directory?
     opendir DIR, $db or die "Can't opendir $db: $!";
     my @files = map "$db/$_", map /(.*)/ && $1, grep !/^\.\.?/, readdir DIR;
     closedir DIR or die "Can't closedir $db: $!";
@@ -118,13 +119,15 @@ sub merge_runs
 {
     my $self = shift;
     my $db = $self->{db};
-    # print "merge_runs from $db/runs/*\n";
+    # print STDERR "merge_runs from $db/runs/*\n";
     # system "ls -al $db/runs";
     return $self unless length $db;
     opendir DIR, "$db/runs" or return $self;
     my @runs = map "$db/runs/$_", grep !/^\.\.?/, readdir DIR;
     closedir DIR or die "Can't closedir $db/runs: $!";
 
+    $self->{changed_files} = {};
+
     # The ordering is important here.  The runs need to be merged in the order
     # they were created.  We're only at a granularity of one second, but that
     # shouldn't be a problem unless a file is altered and the coverage run
@@ -136,8 +139,22 @@ sub merge_runs
         my $r = Devel::Cover::DB->new(base => $self->{base}, db => $run);
         $self->merge($r);
     }
+
     $self->write($db) if @runs;
     rmtree(\@runs);
+
+    if (keys %{$self->{changed_files}})
+    {
+        my $st = Devel::Cover::DB::Structure->new(base => $self->{base});
+        $st->read_all;
+        for my $file (sort keys %{$self->{changed_files}})
+        {
+            # print STDERR "dealing with changed file <$file>\n";
+            $st->delete_file($file);
+        }
+        $st->write($self->{base});
+    }
+
     $self
 }
 
@@ -146,16 +163,38 @@ sub validate_db
     my $self = shift;
 
     # Check validity of the db.  It is valid if the $DB file is there, or if it
-    # is not there but the db directory is empty.
+    # is not there but the db directory is empty, or if there is no db
+    # directory.
     # die if the db is invalid.
 
+    # just warn for now
+    print STDERR "Devel::Cover: $self->{db} is an invalid database\n"
+        unless $self->is_valid;
+
     $self
 }
 
+sub exists
+{
+    my $self = shift;
+    -d $self->{db}
+}
+
 sub is_valid
 {
     my $self = shift;
-    -e "$self->{db}/$DB"
+    return 1 if !-e $self->{db};
+    return 1 if -e "$self->{db}/$DB";
+    opendir my $fh, $self->{db} or return 0;
+    for my $file (readdir $fh)
+    {
+        next if $file eq "." || $file eq "..";
+        next if ($file eq "runs" || $file eq "structure") &&
+                -e "$self->{db}/$file";
+        # warn "found $file in $self->{db}";
+        return 0;
+    }
+    closedir $fh
 }
 
 sub collected
@@ -170,7 +209,7 @@ sub merge
     my ($self, $from) = @_;
 
     # use Data::Dumper; $Data::Dumper::Indent = 1;
-    # print "Merging ", Dumper($self), "From ", Dumper($from);
+    # print STDERR "Merging ", Dumper($self), "From ", Dumper($from);
 
     while (my ($fname, $frun) = each %{$from->{runs}})
     {
@@ -178,7 +217,8 @@ sub merge
         {
             while (my ($name, $run) = each %{$self->{runs}})
             {
-                # print "digests for $file: $digest, $run->{digests}{$file}\n";
+                # print STDERR
+                #     "digests for $file: $digest, $run->{digests}{$file}\n";
                 if ($run->{digests}{$file} && $digest &&
                     $run->{digests}{$file} ne $digest)
                 {
@@ -189,6 +229,7 @@ sub merge
                     delete $run->{digests}{$file};
                     delete $run->{count}  {$file};
                     delete $run->{vec}    {$file};
+                    $self->{changed_files}{$file}++;
                 }
             }
         }
@@ -197,7 +238,7 @@ sub merge
     _merge_hash($self->{runs},      $from->{runs});
     _merge_hash($self->{collected}, $from->{collected});
 
-    return $self;
+    return $self;  # TODO - what's going on here?
 
     # When the database gets big, it's quicker to merge into what's
     # already there.
@@ -210,7 +251,7 @@ sub merge
         $from->{$_} = $self->{$_} unless $_ eq "runs" || $_ eq "collected";
     }
 
-    # print "Giving ", Dumper($from);
+    # print STDERR "Giving ", Dumper($from);
 
     $_[0] = $from;
 }
@@ -394,11 +435,10 @@ sub add_statement
     my %line;
     for my $i (0 .. $#$fc)
     {
-        # print "statement: $i\n";
+        # print STDERR "statement: $i\n";
         my $l = $sc->[$i];
         unless (defined $l)
         {
-            # use Data::Dumper;
             # print STDERR "sc ", scalar @$sc, ", fc ", scalar @$fc, "\n";
             # print STDERR "sc ", Dumper($sc), "fc ", Dumper($fc);
             warn "Devel::Cover: ignoring extra statement\n";
@@ -409,7 +449,7 @@ sub add_statement
         $cc->{$l}[$n][0]  += $fc->[$i];
         $cc->{$l}[$n][1] ||= $uc->{$l}[$n][0][1];
     }
-    # use Data::Dumper; print Dumper $uc;
+    # use Data::Dumper; print STDERR Dumper $uc;
     # use Data::Dumper; print STDERR "cc: ", Dumper $cc;
 }
 
@@ -470,12 +510,16 @@ sub add_subroutine
 {
     my $self = shift;
     my ($cc, $sc, $fc, $uc) = @_;
-    # use Data::Dumper; print STDERR "add_subroutine():\n", Dumper $cc, $sc, $fc, $uc;
+
+    # use Data::Dumper;
+    #     print STDERR "add_subroutine():\n", Dumper $cc, $sc, $fc, $uc;
+
     # $cc = { line_number => [ [ count, sub_name, uncoverable ], [ ... ] ], .. }
     # $sc = [ [ line_number, sub_name ], [ ... ] ]
     # $fc = [ count, ... ]
-    # $cc = { line_number => [ [ ??? ], [ ... ] ], ... }
+    # $uc = { line_number => [ [ ??? ], [ ... ] ], ... }
     # length @$sc == length @$fc
+
     my %line;
     for my $i (0 .. $#$fc)
     {
@@ -488,6 +532,7 @@ sub add_subroutine
             warn "Devel::Cover: ignoring extra subroutine\n";
             return;
         }
+
         my $n = $line{$l}++;
         if (my $a = $cc->{$l}[$n])
         {
@@ -509,7 +554,7 @@ sub uncoverable_files
 {
     my $self = shift;
     my $f = ".uncoverable";
-    (@{$self->{uncoverable}}, $f, glob("~/$f"))
+    (@{$self->{uncoverable_file}}, $f, glob("~/$f"))
 }
 
 sub uncoverable
@@ -526,40 +571,41 @@ sub uncoverable
 
     for my $file ($self->uncoverable_files)
     {
-        open F, $file or next;
+        open my $f, "<", $file or next;
         print STDOUT "Reading uncoverable information from $file\n"
             unless $Devel::Cover::Silent;
-        while (<F>)
+        while (<$f>)
         {
             chomp;
-            my ($file, $crit, $line, $count, $type, $reason) = split " ", $_, 6;
-            push @{$u->{$file}{$crit}{$line}[$count]}, [$type, $reason];
+            my ($file, $crit, $line, $count, $type, $class, $note) =
+                split " ", $_, 7;
+            push @{$u->{$file}{$crit}{$line}[$count]}, [$type, $class, $note];
         }
     }
 
-    # use Data::Dumper; $Data::Dumper::Indent = 1; print Dumper $u;
+    # use Data::Dumper; $Data::Dumper::Indent = 1; print STDERR Dumper $u;
     # Now change the format of the uncoverable information.
 
     for my $file (sort keys %$u)
     {
         # print STDERR "Reading $file\n";
-        unless (open F, $file)
+        open my $fh, "<", $file or do
         {
             warn "Devel::Cover: Can't open file $file: $!\n";
             next;
-        }
+        };
         my $df = Digest::MD5->new;  # MD5 digest of the file
         my %dl;                     # maps MD5 digests of lines to line numbers
         my $ln = 0;                 # line number
-        while (<F>)
+        while (<$fh>)
         {
             # print STDERR "read [$.][$_]\n";
             $dl{Digest::MD5->new->add($_)->hexdigest} = ++$ln;
             $df->add($_);
         }
-        close F;
+        close $fh;
         my $f = $u->{$file};
-        # use Data::Dumper; $Data::Dumper::Indent = 1; print Dumper $f;
+        # use Data::Dumper; $Data::Dumper::Indent = 1; print STDERR Dumper $f;
         for my $crit (keys %$f)
         {
             my $c = $f->{$crit};
@@ -585,7 +631,7 @@ sub uncoverable
         $u->{$df->hexdigest} = delete $u->{$file};
     }
 
-    # use Data::Dumper; $Data::Dumper::Indent = 1; print Dumper $u;
+    use Data::Dumper; $Data::Dumper::Indent = 1; print STDERR Dumper $u;
     $u
 }
 
@@ -595,31 +641,32 @@ sub add_uncoverable
     my ($adds) = @_;
     for my $add (@$adds)
     {
-        my ($file, $crit, $line, $count, $type, $reason) = split " ", $add, 6;
+        my ($file, $crit, $line, $count, $type, $class, $note) =
+            split " ", $_, 7;
         my ($uncoverable_file) = $self->uncoverable_files;
-        open U, ">>", $uncoverable_file
-            or die "Devel::Cover: Can't open $uncoverable_file: $!\n";
 
-        unless (open F, $file)
+        open my $f, "<", $file or do
         {
             warn "Devel::Cover: Can't open $file: $!";
             next;
-        }
-        while (<F>)
+        };
+        while (<$f>)
         {
             last if $. == $line;
         }
         if (defined)
         {
+            open my $u, ">>", $uncoverable_file
+                or die "Devel::Cover: Can't open $uncoverable_file: $!\n";
             my $dl = Digest::MD5->new->add($_)->hexdigest;
-            print U "$file $crit $dl $count $type $reason\n";
+            print $u "$file $crit $dl $count $type $class $note\n";
         }
         else
         {
             warn "Devel::Cover: Can't find line $line in $file.  ",
                  "Last line is $.\n";
         }
-        close F or die "Devel::Cover: Can't close $file: $!\n";
+        close $f or die "Devel::Cover: Can't close $file: $!\n";
     }
 }
 
@@ -639,12 +686,21 @@ sub cover
 
     return $self->{cover} if $self->{cover_valid};
 
-    my %digests;
-    my %files;
+    my %digests;  # mapping of digests to canonical filenames
+    my %files;    # processed files
     my $cover = $self->{cover} = {};
-    my $uncoverable = $self->uncoverable;
+
     my $st = Devel::Cover::DB::Structure->new(base => $self->{base})->read_all;
 
+    my $cr = join "|", @{$self->{all_criteria}};
+    my $uc = qr/(.*)# uncoverable ($cr)(.*)/;  # regex for uncoverable comments
+    my %uncoverable;
+    my %types =
+    (
+        branch    => { true => 0, false => 1 },
+        condition => { left => 0, right => 1, false => 2 },
+    );
+
     # use Data::Dumper; print STDERR "runs: ", Dumper $self->{runs};
     my @runs;
     {
@@ -652,6 +708,7 @@ sub cover
         # TODO - change sort order
         @runs = sort { $b <=> $a } keys %{$self->{runs}};
     }
+
     for my $run (@runs)
     {
         my $r = $self->{runs}{$run};
@@ -671,10 +728,74 @@ sub cover
             print STDERR "Devel::Cover: merging data for $file ",
                          "into $digests{$digest}\n"
                 if !$files{$file}++ && $digests{$digest};
+
+            # Set up data structure to hold coverage being filled in
             my $cf = $cover->{$digests{$digest} ||= $file} ||= {};
+
+            # Look for uncoverable comments
+            open my $fh, "<", $file or do
+            {
+                warn "Devel::Cover: Can't open file $file: $!\n";
+                next;
+            };
+            my @waiting;
+            while (<$fh>)
+            {
+                chomp;
+                # print STDERR "read [$.][$_]\n";
+                next unless /$uc/ || @waiting;
+                if ($2)
+                {
+                    my ($code, $criterion, $info)     = ($1, $2, $3);
+                    my ($count, $class, $note, $type) = (1, "default", "");
+
+                    if ($criterion eq "branch" || $criterion eq "condition")
+                    {
+                        if ($info =~ /^\s*(\w+)(?:\s|$)/)
+                        {
+                            my $t = $1;
+                            $type = $types{$criterion}{$t};
+                            unless (defined $type)
+                            {
+                                warn "Unknown type $t found parsing " .
+                                     "uncoverable $criterion at $file:$.\n";
+                                $type = 999;  # partly magic number
+                            }
+                        }
+                    }
+                    $count = $1 if $info =~ /count:(\d+)/;
+                    $class = $1 if $info =~ /class:(\w+)/;
+                    $note  = $1 if $info =~ /note:(.+)/;
+
+                    # no warnings "uninitialized";
+                    # warn "pushing $criterion, $count, $type, $class, $note";
+
+                    push @waiting,
+                         [$criterion, $count - 1, $type, $class, $note];
+
+                    next unless $code =~ /\S/;
+                }
+
+                # found what we are waiting for
+                while (my $w = shift @waiting)
+                {
+                    my ($criterion, $count, $type, $class, $note) = @$w;
+                    push @{$uncoverable{$digest}{$criterion}{$.}[$count]},
+                         [$type, $class, $note];
+                }
+            }
+            close $fh;
+
+            warn scalar @waiting,
+                 " unmatched uncoverable comments not found at end of $file\n"
+                if @waiting;
+
+            # TODO - read in and merge $self->uncoverable;
+            # use Data::Dumper; print Dumper \%uncoverable;
+
             # print STDERR "st ", Dumper($st),
                          # "f  ", Dumper($f),
-                         # "uc ", Dumper($uncoverable->{$digest});
+                         # "uc ", Dumper($uncoverable{$digest});
             while (my ($criterion, $fc) = each %$f)
             {
                 my $get = "get_$criterion";
@@ -684,7 +805,7 @@ sub cover
                 my $cc  = $cf->{$criterion} ||= {};
                 my $add = "add_$criterion";
                 # print STDERR "$add():\n", Dumper $cc, $sc, $fc;
-                $self->$add($cc, $sc, $fc, $uncoverable->{$digest}{$criterion});
+                $self->$add($cc, $sc, $fc, $uncoverable{$digest}{$criterion});
                 # print STDERR "--> $add():\n", Dumper $cc;
                 # $cc - coverage being filled in
                 # $sc - structure information
@@ -886,11 +1007,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.59 - 23rd August 2006
+Version 0.66 - 12th April 2010
 
 =head1 LICENCE
 
-Copyright 2001-2006, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
