@@ -1,4 +1,4 @@
-# Copyright 2001-2005, Paul Johnson (pjcj@cpan.org)
+# Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
 
 # This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -10,26 +10,28 @@ package Devel::Cover::Report::Html_basic;
 use strict;
 use warnings;
 
-our $VERSION = "0.55";
+our $VERSION = "0.66";
 
-use Devel::Cover::DB 0.55;
+use Devel::Cover::DB 0.66;
+use Devel::Cover::Web 0.66 "write_file";
 
 use Getopt::Long;
 use Template 2.00;
 
+my ($Have_highlighter, $Have_PPI, $Have_perltidy);
+
+BEGIN
+{
+    eval "use PPI; use PPI::HTML;";
+    $Have_PPI = !$@;
+    eval "use Perl::Tidy";
+    $Have_perltidy = !$@;
+    $Have_highlighter = $Have_PPI || $Have_perltidy;
+}
+
 my $Template;
 my %R;
 
-sub print_stylesheet
-{
-    my $file = "$R{options}{outputdir}/cover.css";
-    open CSS, '>', $file or return;
-    my $p = tell DATA;
-    print CSS <DATA>;
-    seek DATA, $p, 0;
-    close CSS;
-}
-
 sub oclass
 {
     my ($o, $criterion) = @_;
@@ -78,7 +80,7 @@ sub print_summary
     my $vars =
     {
         R     => \%R,
-        files => [ grep($R{db}->summary($_), @{$R{options}{file}}), "Total" ],
+        files => [ "Total", grep $R{db}->summary($_), @{$R{options}{file}} ],
     };
 
     my $html = "$R{options}{outputdir}/$R{options}{option}{outputfile}";
@@ -87,15 +89,81 @@ sub print_summary
     $html
 }
 
+sub _highlight_ppi {
+    my @all_lines = @_;
+    my $code = join "", @all_lines;
+    my $document = PPI::Document->new(\$code);
+    my $highlight = PPI::HTML->new(line_numbers => 1);
+    my $pretty =  $highlight->html($document);
+
+    my $split = '<span class="line_number">';
+
+    no warnings "uninitialized";
+
+    # turn significant whitespace into &nbsp;
+    @all_lines = map {
+        $_ =~ s{</span>( +)}{"</span>" . ("&nbsp;" x length($1))}e;
+        "$split$_";
+    } split /$split/, $pretty;
+
+    # remove the line number
+    @all_lines = map {
+        s{<span class="line_number">.*?</span>}{}; $_;
+    } @all_lines;
+    @all_lines = map {
+        s{<span class="line_number">}{}; $_;
+    } @all_lines;
+
+    # remove the BR
+    @all_lines = map {
+        s{<br>$}{}; $_;
+    } @all_lines;
+    @all_lines = map {
+        s{<br>\n</span>}{</span>}; $_;
+    } @all_lines;
+
+    shift @all_lines if $all_lines[0] eq "";
+    return @all_lines;
+}
+
+sub _highlight_perltidy {
+    my @all_lines = @_;
+    my @coloured = ();
+
+    Perl::Tidy::perltidy(
+        source      => \@all_lines, 
+        destination => \@coloured, 
+        argv        => '-html -pre -nopod2html', 
+        stderr      => '-',
+        errorfile   => '-',
+    );
+
+    # remove the PRE
+    shift @coloured;
+    pop @coloured;
+    @coloured = grep { !/<a name=/ } @coloured;
+
+    return @coloured
+}
+
+*_highlight = $Have_PPI      ? \&_highlight_ppi
+            : $Have_perltidy ? \&_highlight_perltidy
+            : sub {};
+
 sub print_file
 {
     my @lines;
     my $f = $R{db}->cover->file($R{file});
 
-    open F, $R{file} or warn("Unable to open $R{file}: $!\n"), next;
-    LINE: while (defined(my $l = <F>))
+    open F, $R{file} or warn("Unable to open $R{file}: $!\n"), return;
+    my @all_lines = <F>;
+
+    @all_lines = _highlight(@all_lines) if $Have_highlighter;
+
+    my $linen = 1;
+    LINE: while (defined(my $l = shift @all_lines))
     {
-        my $n = $.;
+        my $n  = $linen++;
         chomp $l;
 
         my %criteria;
@@ -180,6 +248,9 @@ sub print_branches
         for my $b (@{$branches->location($location)})
         {
             $count++;
+            my $text = $b->text;
+            ($text) = _highlight($text) if $Have_highlighter;
+
             push @branches,
                 {
                     number     => $count == 1 ? $location : "",
@@ -190,7 +261,7 @@ sub print_branches
                                              "branch") },
                             0 .. $b->total - 1
                     ],
-                    text       => $b->text,
+                    text       => $text,
                 };
         }
     }
@@ -218,6 +289,9 @@ sub print_conditions
         {
             $count{$c->type}++;
             # print "-- [$count{$c->type}][@{[$c->text]}]}]\n";
+            my $text = $c->text;
+            ($text) = _highlight($text) if $Have_highlighter;
+
             push @{$r{$c->type}},
                 {
                     number     => $count{$c->type} == 1 ? $location : "",
@@ -229,7 +303,7 @@ sub print_conditions
                                              "condition") },
                             0 .. $c->total - 1
                     ],
-                    text       => $c->text,
+                    text       => $text,
                 };
         }
     }
@@ -301,7 +375,7 @@ sub get_options
 {
     my ($self, $opt) = @_;
     $opt->{option}{outputfile} = "coverage.html";
-    die "Bad option" unless
+    die "Invalid command line options" unless
         GetOptions($opt->{option},
                    qw(
                        outputfile=s
@@ -323,6 +397,12 @@ sub report
     %R =
     (
         db      => $db,
+        date    => do
+        {
+            my ($sec, $min, $hour, $mday, $mon, $year) = localtime;
+            sprintf "%04d-%02d-%02d %02d:%02d:%02d",
+                    $year + 1900, $mon + 1, $mday, $hour, $min, $sec
+        },
         options => $options,
         showing => [ grep $options->{show}{$_}, $db->criteria ],
         headers =>
@@ -345,7 +425,7 @@ sub report
         get_summary => \&get_summary,
     );
 
-    print_stylesheet;
+    write_file $R{options}{outputdir}, "all";
     my $html = print_summary;
 
     for (@{$options->{file}})
@@ -368,7 +448,7 @@ package Devel::Cover::Report::Html_basic::Template::Provider;
 use strict;
 use warnings;
 
-our $VERSION = "0.55";
+our $VERSION = "0.66";
 
 use base "Template::Provider";
 
@@ -383,29 +463,25 @@ sub fetch
 }
 
 $Templates{html} = <<'EOT';
+<!DOCTYPE html
+     PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
+     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
+<html xmlns="http://www.w3.org/1999/xhtml">
 <!--
-
-This file was generated by Devel::Cover Version 0.55
-
-Devel::Cover is copyright 2001-2002, Paul Johnson (pjcj@cpan.org)
-
-Devel::Cover is free.  It is licensed under the same terms as Perl itself.
-
+This file was generated by Devel::Cover Version 0.66
+Devel::Cover is copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
+Devel::Cover is free. It is licensed under the same terms as Perl itself.
 The latest version of Devel::Cover should be available from my homepage:
 http://www.pjcj.net
-
 -->
-
-<?xml version="1.0" encoding="utf-8"?>
-<!DOCTYPE html
-    PUBLIC "-//W3C//DTD XHTML Basic 1.0//EN"
-    "http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd">
-<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
-    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"></meta>
+    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"></meta>
     <meta http-equiv="Content-Language" content="en-us"></meta>
     <link rel="stylesheet" type="text/css" href="cover.css"></link>
-    <title> [% title %] </title>
+    <script type="text/javascript" src="common.js"></script>
+    <script type="text/javascript" src="css.js"></script>
+    <script type="text/javascript" src="standardista-table-sorting.js"></script>
+    <title> [% title || "Coverage Summary" %] </title>
 </head>
 <body>
     [% content %]
@@ -416,7 +492,7 @@ EOT
 $Templates{header} = <<'EOT';
 <table>
     <tr>
-        <th colspan=4>[% R.file %]</th>
+        <th colspan="4">[% R.file %]</th>
     </tr>
     <tr class="hblank"><td class="dblank"></td></tr>
     <tr>
@@ -431,7 +507,7 @@ $Templates{header} = <<'EOT';
             <td class="h">[% criterion %]</td>
             <td>[% vals.covered %]</td>
             <td>[% vals.total %]</td>
-            <td class="[% vals.class %]" title="[% vals.details %]">
+            <td [% IF vals.class %]class="[% vals.class %]" [% END %]title="[% vals.details %]">
                 [% IF vals.link.defined %]
                     <a href="[% vals.link %]"> [% vals.pc %] </a>
                 [% ELSE %]
@@ -450,20 +526,27 @@ $Templates{summary} = <<'EOT';
 <h1> Coverage Summary </h1>
 <table>
     <tr>
-        <td class="h" align="right">Database:</td>
-        <td>[% R.db.db %]</td>
+        <td class="sh" align="right">Database:</td>
+        <td class="sv" align="left">[% R.db.db %]</td>
+    </tr>
+    <tr>
+        <td class="sh" align="right">Report date:</td>
+        <td class="sv" align="left">[% R.date %]</td>
     </tr>
 </table>
 <div><br></br></div>
-<table>
-    <tr>
-    <th> file </th>
-    [% FOREACH header = R.headers %]
-        <th> [% header %] </th>
-    [% END %]
-    <th> total </th>
-    </tr>
+<table class="sortable" id="coverage_table">
+    <thead>
+        <tr>
+            <th> file </th>
+            [% FOREACH header = R.headers %]
+                <th> [% header %] </th>
+            [% END %]
+            <th> total </th>
+        </tr>
+    </thead>
 
+    <tfoot>
     [% FOREACH file = files %]
         <tr align="center" valign="top">
             <td align="left">
@@ -494,7 +577,13 @@ $Templates{summary} = <<'EOT';
                 [% vals.pc %]
             </td>
         </tr>
+
+        [% IF file == "Total" %]
+            </tfoot>
+            <tbody>
+        [% END %]
     [% END %]
+    </tbody>
 </table>
 
 [% END %]
@@ -525,7 +614,9 @@ $Templates{file} = <<'EOT';
 
     [% FOREACH line = lines %]
         <tr>
-            <td [% IF line.number %] class="h" [% END %]>[% line.number %]</td>
+            <td [% IF line.number %] class="h" [% END %]>
+                <a [% IF line.number != '&nbsp;' %]name="[% line.number %]"[% END %]>[% line.number %]</a>
+            </td>
             [% FOREACH cr = line.criteria %]
                 <td [% IF cr.class %] class="[% cr.class %]" [% END %]>
                     [% IF cr.link.defined %] <a href="[% cr.link %]"> [% END %]
@@ -665,7 +756,8 @@ statistics
 =head1 DESCRIPTION
 
 This module provides a HTML reporting mechanism for coverage data.  It
-is designed to be called from the C<cover> program.
+is designed to be called from the C<cover> program. It will add syntax
+highlighting if C<PPI::HTML> or C<Perl::Tidy> is installed.
 
 =head1 SEE ALSO
 
@@ -677,11 +769,11 @@ Huh?
 
 =head1 VERSION
 
-Version 0.55 - 22nd September 2005
+Version 0.66 - 12th April 2010
 
 =head1 LICENCE
 
-Copyright 2001-2005, Paul Johnson (pjcj@cpan.org)
+Copyright 2001-2010, Paul Johnson (pjcj@cpan.org)
 
 This software is free.  It is licensed under the same terms as Perl itself.
 
@@ -689,95 +781,3 @@ The latest version of this software should be available from my homepage:
 http://www.pjcj.net
 
 =cut
-
-package Devel::Cover::Report::Html_basic;
-
-__DATA__
-
-/* Stylesheet for Devel::Cover HTML reports */
-
-/* You may modify this file to alter the appearance of your coverage
- * reports. If you do, you should probably flag it read-only to prevent
- * future runs from overwriting it.
- */
-
-/* Note: default values use the color-safe web palette. */
-
-body {
-    font-family: sans-serif;
-}
-
-h1 {
-    text-align : center;
-    background-color: #cc99ff;
-    border: solid 1px #999999;
-    padding: 0.2em;
-    -moz-border-radius: 10px;
-}
-
-a {
-    color: #000000;
-}
-a:visited {
-    color: #333333;
-}
-
-table {
-    border-spacing: 1px;
-}
-tr {
-    text-align : center;
-    vertical-align: top;
-}
-th,.h,.hh {
-    background-color: #cccccc;
-    border: solid 1px #333333;
-    padding-left:  0.2em;
-    padding-right: 0.2em;
-    width: 2.5em;
-    -moz-border-radius: 4px;
-}
-.hh {
-    width: 25%;
-}
-td {
-    border: solid 1px #cccccc;
-    -moz-border-radius: 4px;
-}
-.hblank {
-    height: 0.5em;
-}
-.dblank {
-    border: none;
-}
-
-/* source code */
-pre,.s {
-    text-align: left;
-    font-family: monospace;
-    white-space: pre;
-    padding: 0.2em 0.5em 0em 0.5em;
-}
-
-/* Classes for color-coding coverage information:
- *   c0  : path not covered or coverage < 75%
- *   c1  : coverage >= 75%
- *   c2  : coverage >= 90%
- *   c3  : path covered or coverage = 100%
- */
-.c0 {
-    background-color: #ff9999;
-    border: solid 1px #cc0000;
-}
-.c1 {
-    background-color: #ffcc99;
-    border: solid 1px #ff9933;
-}
-.c2 {
-    background-color: #ffff99;
-    border: solid 1px #cccc66;
-}
-.c3 {
-    background-color: #99ff99;
-    border: solid 1px #009900;
-}
